<?php
/*
	Plugin Name: WooCommerce PagSeguro Gateway
	Plugin URI: http://www.vibemidia.com/woocommerce-pagseguro
	Description: Integração PagSeguro para WooCommerce. Siga as <a href="http://www.vibemidia.com/woocommerce-pagseguro">instruções de uso</a>. <strong>Adquira o plugin <a href="http://woocommerce.com.br/plugin/pagamento-digital-plugin-para-woocommerce/" target="_new">Pagamento Digital</a>, <a href="http://woocommerce.com.br/plugin/moip-plugin-woocommerce/" target="_new">MoIP</a> e <a href="http://woocommerce.com.br/plugin/correios-plugin-para-woocommerce/" target="_new">Correios</a></strong>.
	Version: 1.3
	Author: John-Henrique 
	Author URI: http://johnhenrique.com
	Requires at least: 3.1
	Tested up to: 3.5.1
	Text domain: WC_PagSeguro
*/

/*  Copyright 2011 John-Henrique 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Required functions
 
if ( ! function_exists( 'woothemes_queue_update' ) )
	require_once( 'woo-includes/woo-functions.php' );
*/
/**
 * Plugin updates
 */
//woothemes_queue_update( plugin_basename( __FILE__ ), '58b516c415a082f44cfc46eb2e844b2a', '18641' );

add_action('plugins_loaded', 'woocomerce_gateway_pagseguro', 0);

function woocomerce_gateway_pagseguro() {
 
	if ( !class_exists( 'WC_Payment_Gateway' ) ) return;
	
	load_plugin_textdomain('WC_PagSeguro', false, basename( dirname( __FILE__ ) ) . '/languages' );
	
	add_filter('woocommerce_payment_gateways', 'add_pagseguro_gateway' );
	
	class WC_Gateway_Pagseguro extends WC_Payment_Gateway {
		
		public function __construct() { 
			global $woocommerce;
			
	        $this->id			= 'pagseguro';
	        $this->method_title = 'PagSeguro';
			$this->icon 		= WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/pagseguro.png';
			$this->versao		= '1.3';
	        $this->has_fields 	= false;
	        $this->liveurl 		= 'https://pagseguro.uol.com.br/v2/checkout/payment.html';
	        
	        
			// Load the form fields.
			$this->init_form_fields();
			
			// Load the settings.
			$this->init_settings();
			
			// Define user set variables
			$this->title 		= $this->settings['title'];
			$this->description 	= $this->settings['description'];
			$this->email 		= $this->settings['email'];
			$this->token 		= $this->settings['token'];
			$this->debug		= $this->settings['debug'];	
			$this->notificationCode	= (isset($this->settings['notification_code']))?$this->settings['notification_code']:'notification_code';
			
			
			// order prefix
			$this->prefix 		= (isset($this->settings['prefix']))?$this->settings['prefix']:'';
		
			// Logs
			if ($this->debug=='yes') $this->log = new WC_Logger();
			

			if ( ! $this->is_valid_currency() || ! $this->are_credentials_set() )
				$this->enabled = false;
			
				
				
			add_action('init', array(&$this, 'fix_url_order_received' ));
				
			// Actions
			add_action('woocommerce_receipt_pagseguro',	array( $this, 'receipt_page' ) );
			add_action( 'woocommerce_update_options_payment_gateways_'. $this->id, array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_api_'. strtolower( get_class( $this ) ), array( $this, 'notificacao' ) );

			// Actions
			add_action('woocommerce_receipt_pagseguro',	array(&$this, 'receipt_page' ));
			
			
			// support to subscription plugin
			//$this->supports = array( 'subscriptions', 'products' );
	    } 
	    
	    
		/**
	     * Initialise Gateway Settings Form Fields
	     */
	    function init_form_fields() {
	    
	    	$this->form_fields = array(
				'enabled' => array(
								'title' => __( 'Enable/Disable', 'WC_PagSeguro' ), 
								'type' => 'checkbox', 
								'label' => __( 'Enable PagSeguro standard', 'WC_PagSeguro' ), 
								'default' => 'yes'
							), 
				'title' => array(
								'title' => __( 'Title', 'WC_PagSeguro' ), 
								'type' => 'text', 
								'description' => __( 'This controls the title which the user sees during checkout.', 'WC_PagSeguro' ), 
								'default' => __( 'PagSeguro', 'WC_PagSeguro' )
							),
				'description' => array(
								'title' => __( 'Description', 'WC_PagSeguro' ), 
								'type' => 'textarea', 
								'description' => __( 'This controls the description which the user sees during checkout.', 'WC_PagSeguro' ), 
								'default' => __("Pay securely via PagSeguro using your Credit Card or bank account.", 'WC_PagSeguro')
							),
				'email' => array(
								'title' => __( 'PagSeguro Email', 'WC_PagSeguro' ), 
								'type' => 'text', 
								'description' => __( 'Please enter your PagSeguro email address; this is needed in order to take payment.', 'WC_PagSeguro' ), 
								'default' => ''
							),
				'token' => array(
								'title' => __( 'PagSeguro Token', 'WC_PagSeguro' ), 
								'type' => 'text', 
								'description' => __( 'Please enter your PagSeguro token; this is needed in order to take payment. See <a href="https://pagseguro.uol.com.br/integracao/token-de-seguranca.jhtml" target="_blank">Token of security</a>', 'WC_PagSeguro' ), 
								'default' => ''
							),
				'prefix' => array(
								'title' => __( 'Order prefix', 'WC_PagSeguro' ), 
								'type' => 'text', 
								'description' => __( 'Choose a prefix to orders. Eg WC this case order ID is would "WC-5126452".', 'WC_PagSeguro' ), 
								'size' => 2, 
								'default' => 'WC'
							),
				'notification_code' => array(
								'title' => __( 'What is your transaction code?', 'WC_PagSeguro' ), 
								'type' => 'text', 
								'description' => __( 'This is used in redirect page and return of data. Change only if you change in your PagSeguro account. See <a href="https://pagseguro.uol.com.br/integracao/pagina-de-redirecionamento.jhtml" target="_blank">Transaction code</a>', 'WC_PagSeguro' ), 
								'default' => 'notification_code'
							),
				'debug' => array(
								'title' => __( 'Enable/Disable Log', 'WC_PagSeguro' ), 
								'type' => 'checkbox', 
								'description' => __( 'Enable logging (<code>woocommerce/logs/pagseguro.txt</code>)', 'WC_PagSeguro' ), 
								'default' => ''
							),
				
				);
	    
	    } // End init_form_fields()
	    
	    
		/**
		 * Admin Panel Options 
		 * - Options for bits like 'title' and availability on a country-by-country basis
		 *
		 * @since 1.0.0
		 */
		public function admin_options() {
	
	    	?>
	    	<h3><?php echo __('PagSeguro Gateway (API v2)', 'WC_PagSeguro'); ?></h3>
	    	<p><?php echo __('PagSeguro works by sending the user to PagSeguro Gateway to enter their payment information. This Payment Gateway works only API v2 and you need make changes in <a href="https://pagseguro.uol.com.br/integracao/notificacao-de-transacoes.jhtml" target="_blank">Notifications of Transactions</a>', 'WC_PagSeguro'); ?></p>
	    	<table class="form-table">
				<?php if ( ! $this->is_valid_currency() ) : ?>
					<div class="inline error">
						<p><strong><?php _e( 'Gateway Disabled', 'WC_PagSeguro' ); ?></strong>: <?php _e( 'PagSeguro does not support your store\'s currency. You need to select the currency of Brazil Real.', 'WC_PagSeguro' ); ?></p>
					</div>
				<?php endif; ?>
				
				<?php if ( ! $this->are_credentials_set() ) : ?>
					<div class="inline error">
						<p><strong><?php _e( 'Gateway Disabled', 'WC_PagSeguro' ); ?></strong>: <?php _e( 'You must give the token of your account email.', 'WC_PagSeguro' ); ?></p>
					</div>
				<?php endif; ?>
	    	<?php
	    		// Generate the HTML For the settings form.
	    		$this->generate_settings_html();
	    	?>
			</table><!--/.form-table-->
	    	<?php
	    } // End admin_options()
	    
	    
	    
		/**
		 * Check if PagSeguro can be used with the store's currency.
		 * For now only work with Real of Brazil, but...
		 *
		 * @since 1.0
		 */
		function is_valid_currency() {
			if ( !in_array( get_option( 'woocommerce_currency' ), array( 'BRL' ) ) )
				return false;
			else
				return true;
		}
		


		/**
		 * Check if PagSeguro Credentials are set
		 *
		 * @since 1.0
		 */
		function are_credentials_set() {
			if( empty( $this->email ) || empty( $this->token ) )
				return false;
			else
				return true;
		}
	    
	    
	    /**
		 * There are no payment fields for PagSeguro, but we want to show the description if set.
		 **/
	    function payment_fields() {
	    	if ($this->description) echo wpautop(wptexturize($this->description));
	    }



		/**
		 * Generate the PagSeguro button link
		 **/
	    public function generate_pagseguro_form_API( $order_id ) {
			global $woocommerce;
			
			$order = new WC_Order( $order_id );
			
			// give me, please =)   (receive a token)
			$token = $this->request_token_API( $order );
			
			
			// show error code if exists
			$codigo = 0;
			if( isset( $woocommerce->session->PSerro ) && !empty( $woocommerce->session->PSerro ) ){
				$codigo = $woocommerce->session->PSerro;
			}
			
			
			// token is false
			if( $token == false ) {

				$html  ="<h3>". __("Sorry, an error has occurred", 'WC_PagSeguro') ."</h3>";
				$html .="<p>". __("Try again, if the problem persists try to choose another payment method or contact the service.", 'WC_PagSeguro') ."</p>";
				$html .="<p>Código: ". $codigo ."</p>";
				
			    if ($this->debug=='yes') $this->log->add( $this->id, 'Token inválido/falso: '. $codigo ."<br>". $token );

				
			} else {
				
				$js = '
				jQuery(function(){
					jQuery("body").block(
						{ 
							message: "'.__('Thank you for your order. We are now redirecting you to PagSeguro to make payment.', 'WC_PagSeguro').'", 
							overlayCSS: 
							{ 
								background: "#fff", 
								opacity: 0.6 
							},
							css: { 
						        padding:        20, 
						        textAlign:      "center", 
						        color:          "#555", 
						        border:         "3px solid #aaa", 
						        backgroundColor:"#fff", 
						        cursor:         "wait",
						        lineHeight:		"32px"
						    } 
						});
					jQuery("#submit_pagseguro_payment_form").click();
				});';

				if ( function_exists( 'wc_enqueue_js' ) ) {
					wc_enqueue_js( $js );
				} else {
					$woocommerce->add_inline_js( $js );
				}
				
				// HTML to payment
				$html = '<form action="'.esc_url( $token ).'" method="post" id="pagseguro_payment_form">
							<input type="submit" class="button-alt" id="submit_pagseguro_payment_form" value="'.__('Pay via PagSeguro', 'WC_PagSeguro').'" /> <a class="button cancel" href="'.esc_url( $order->get_cancel_order_url() ).'">'.__('Cancel order &amp; restore cart', 'WC_PagSeguro').'</a>
						</form>';
				
				
				if ($this->debug=='yes') $this->log->add( $this->id, 'Formulário de pagamento gerado com sucesso.' );
			}
			
			return $html;
		}
		
		
		/**
		 * Process the payment and return the result
		 * checkout
		 * checkout page
		 * 
		 **/
		function process_payment( $order_id = 0 ) {
			global $woocommerce;
			
			$order = new WC_Order( $order_id );

			
			if($this->debug=='yes') $this->log->add(  $this->id, 'WooCommerce '. WOOCOMMERCE_VERSION );
			if($this->debug=='yes') $this->log->add(  $this->id, 'WooCommerce '. get_class( $this ) .' '. $this->versao );
			
			
			// clean cart
			$woocommerce->cart->empty_cart();

			// Empty awaiting payment session
			unset( $woocommerce->session->order_awaiting_payment );

			return array(
				'result' 	=> 'success',
				'redirect'	=> $order->get_checkout_payment_url( true )
			);
		}
		
		
		/**
		 * Checkout -> Pay
		 * Payment page, redirect to gateway
		 **/
		function receipt_page( $order ) {
			echo '<p>'.__('Thank you for your order, please click the button below to pay with PagSeguro.', 'WC_PagSeguro').'</p>';
			echo $this->generate_pagseguro_form_API( $order );
		}
		
		
		/**
		 * Ajuda a identificar o status de pagamento
		 * @see https://pagseguro.uol.com.br/v2/guia-de-integracao/api-de-notificacoes.html#v2-item-api-de-notificacoes-status-da-transacao
		 * @param Integer $status status de pagamento retornado
		 * @return Array status de pagamento convertido para o WooCommerce
		 */
		public function status_helper( $status = null, $order = null ) {
			global $woocommerce;
			
			switch ( $status ){
				
				case 1:
					//$arrStatus['status'] = "pending"; // aguardando pagamento
					$arrStatus['status'] = "on-hold"; // aguardando pagamento
					$arrStatus['log'] = __('Payment pending, Waiting payment confirmation', 'WC_PagSeguro'); // aguardando pagamento
					
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
		    		
					break;
				
				case 2:
					$arrStatus['status'] = "pending"; // em analise
					$arrStatus['log'] = __('Payment in analysis, Waiting payment confirmation', 'WC_PagSeguro');
		    		
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
		    		
					break;
				
				case 3:
					$arrStatus['status'] = "completed"; // paga
					$arrStatus['log'] = __('Manual confirmation of payment, Check payment confirmation', 'WC_PagSeguro');
					
		    		// change payment status
		    		$order->payment_complete();
		    		
					break;
				
				case 4:// disponivel
					$arrStatus['status'] = "completed"; // disponivel
					$arrStatus['log'] = __('Payment completed', 'WC_PagSeguro');
					
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
		    		
					break;
				
				case 5: // disputa
					$arrStatus['status'] = "on-hold"; 
					$arrStatus['log'] = __('Failed on process of negociation (dispute)', 'WC_PagSeguro');
					
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
		    		
					break;
				
				case 6:
					$arrStatus['status'] = "refunded"; // devolvida
					$arrStatus['log'] = __('Payment refunded', 'WC_PagSeguro');
					
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
		    		
					break;
				
				case 7:
					$arrStatus['status'] = "cancelled"; // cancelada
					$arrStatus['log'] = __('Order cancelled', 'WC_PagSeguro');
					
		    		// cancel this order 
		    		$order->cancel_order( $order->id );
		    		
					break;
					
				// improvavel mas nao custa prevenir
				default:
					$arrStatus['status'] = 'pending'; 
					$arrStatus['log'] = __('Payment pending, Waiting payment confirmation', 'WC_PagSeguro');
					
					// add note to control
					$order->add_order_note( $arrStatus['log'] );
			}
			
			$order->update_status( $arrStatus['status'], $arrStatus['log'] );
	    	
	    	// What is status in this case?
	    	if ($this->debug=='yes') $this->log->add( $this->id, "Status \n Status found: ". $arrStatus['status'] ."\n Status source: ". $status );
			
			return $arrStatus;
		}
		
		
		
		/**
		 * This a hack fix to dinamic URL used in PagSeguro
		 * 
		 * Until this day PagSeguro no have supports to 
		 * query strings in redirectURL
		 * 
		 * @since 2012-04-24
		 */
		public function fix_url_order_received(){
			global $woocommerce;
			
			
			// if notificationCode and order exists and have a value
			if( isset( $_GET[$this->notificationCode] ) && !empty( $_GET[$this->notificationCode] ) && isset( $_GET['order'] ) && !empty( $_GET['order'] ) ){
				
			    // this order exists?
			    $order = new WC_Order( (int) $_GET['order'] );
				
				/**
		    	 * PagSeguro no support URLs with query strings
		    	 * This is a simple hack to fix this problem
		    	 * 
		    	 * Verify the transaction code and redirect 
		    	 * to URL with order_id and key
		    	 */
		    	if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
                	$redirect = $order->get_checkout_order_received_url();
                } else {
                	$url_retorno = add_query_arg( add_query_arg( 'order', $order->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) ) .';key='. $order->order_key;
                }

		    	wp_redirect( $redirect );
			}
		}
		
		
		
		/**
		 * Notification of status payments
		 *
		 */
		function notificacao() {
			global $woocommerce;
			
			// who are you?
			if(isset($_POST['notificationType']) && $_POST['notificationType'] == 'transaction') {
				
				

				
				if ($this->debug=='yes') $this->log->add( $this->id, 'Notificação de pagamento $_POST[notificationCode] '. print_r( $_POST['notificationCode'], true ) );
				
				// Fix return of data in new version of PagSeguro API 2.0
				$url = "https://ws.pagseguro.uol.com.br/v2/transactions/notifications/" . $_POST['notificationCode'] . "?email=" . $this->email . "&token=" . $this->token;
				
			    if ($this->debug=='yes') $this->log->add( $this->id, 'Valor do this->notificationCode '. $this->notificationCode );
			    
			    
			    $curl = curl_init($url);
			    curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			    $transaction= curl_exec($curl);
			    curl_close($curl);
			
			    /**
			     * if any errors occur 
			     * kill this process
			     */
			    if($transaction == 'Unauthorized'){
			    	
			    	if ($this->debug=='yes') $this->log->add( $this->id, 'Verificação não autorizada (valores inválidos)' );
			    	
			    	$woocommerce->add_error(__('There was an error during the payment process, please try again.', 'WC_PagSeguro'));
			        exit;
			    }
			    
			    $transaction = simplexml_load_string($transaction);
			    
			    // short cut
			    $order_code 	= $transaction->code;
			    $order_ID 		= end( explode( '-', $transaction->reference ));
			    $order_status 	= $transaction->status;
			    $order_total 	= $transaction->grossAmount; // sem desconto
			    $order_desconto = $transaction->netAmount; // com desconto
			    
			    // this order exists?
			    $pedido = new WC_Order( (int) $order_ID );
			    
			    if ($this->debug=='yes') $this->log->add( $this->id, 'Notificação do pedido #'. print_r( $pedido->id, true ) );
			    
			    
			    /**
			     * For security reasons we check if the application 
			     * exists and that the total order value is the 
			     * same as the amount charged in store
			     */
		    	// Is order total correct?
		    	if ($this->debug=='yes') $this->log->add( $this->id, 'Preço do pedido: '. $pedido->order_total .' - Preço do retorno: '. $order_total );
		    	
			    if( $order_total == $pedido->order_total ) {
			    	
			    	// what a is this?
			    	$arrStatus = $this->status_helper( $order_status, $pedido );
			    	
			    	
			    	/**
			    	 * PagSeguro no support URLs with query strings
			    	 * This is a simple hack to fix this problem
			    	 * 
			    	 * Verify the transaction code and redirect 
			    	 * to URL with order_id and key
			    	 */
			    	if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
	                	$redirect = $pedido->get_checkout_order_received_url();
	                } else {
	                	$url_retorno = add_query_arg( add_query_arg( 'order', $order->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) ) .';key='. $order->order_key;
	                }

			    	wp_redirect( $redirect );
			    	
			    } else {
			    	
			    	if ($this->debug=='yes') $this->log->add( $this->id, __('There was an error during the payment process, this order no exists.', 'WC_PagSeguro') );
			    	
			    	// this order no exists or was modified
			    	$woocommerce->add_error(__('There was an error during the payment process, this order no exists.', 'WC_PagSeguro'));
			    }

			}
		}
		
		
		
		protected function order_itens( $pedido ){
			
			$item_loop = 0;
			$xml = '';
			
			
			// Percorrendo Array de itens
			foreach ( $pedido->get_items( ) as $item_pedido ){
				
				// verificando se existem itens
				if( $item_pedido['qty'] ){
				
					$item_loop++;
					
					
					// Preço do produto
					$item_preco = $pedido->get_item_subtotal( $item_pedido, false );
					
					$xml .= '
					<item>
						<id>'. $item_loop .'</id>
						<description>'. $item_pedido['name'] .'</description>
						<amount>'. number_format( $item_preco, 2, '.', '' ) .'</amount>
						<quantity>'. $item_pedido['qty'] .'</quantity>
						<weight>0</weight>
					</item>
					';
				}
			}
			
			return $xml;
		}
		
		
		
		
		public function request_token_API( $order ) {
			global $woocommerce;
	
			$telefone_tag = '';
			$order->billing_phone = str_replace(array('(', '-', ' ', ')'), '', $order->billing_phone);
			$ddd = substr($order->billing_phone, 0, 2 );
			$telefone = substr($order->billing_phone, 2 );
			
			/**
			 * Garantindo que o nó <phone> esta correto 
			 * só será valido se houver 10 digitos
			 */
			if( strlen( $order->billing_phone ) == 10 ){
				$telefone_tag = "<phone><areacode>". $ddd ."</areacode><number>". $telefone ."</number></phone>";
			}
			
			
			// no support to query string
			if ( method_exists( $order, 'get_checkout_order_received_url' ) ) {
            	$url_retorno = $order->get_checkout_order_received_url();
            } else {
            	$url_retorno = add_query_arg( add_query_arg( 'order', $order->id, get_permalink( get_option( 'woocommerce_thanks_page_id' ) ) ) ) .';key='. $order->order_key;
            }
			
			
			// no support to query string
			$url_notificacao = home_url() ."/?wc-api=". get_class( $this );
			if ($this->debug=='yes') $this->log->add( $this->id, "URL de notificação: ". $url_notificacao );
			
			

			$url = "https://ws.pagseguro.uol.com.br/v2/checkout/?email=" . $this->email . "&token=" . $this->token;
			$xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>
			    <checkout>
			        <currency>BRL</currency>
			        <redirectURL>". $url_retorno ."</redirectURL>
			        <notificationURL>". $url_notificacao ."</notificationURL>
			        <items>
			        	". /*$this->order_itens( $order )*/ null ."
						<item>
							<id>1</id>
							<description>Pagamento do pedido #". $order->id ."</description>
							<amount>". number_format( $order->order_total, 2, '.', '' ) ."</amount>
							<quantity>1</quantity>
							<weight>0</weight>
						</item>
			        </items>
			        <reference>". $this->prefix .'-'. $order->id ."</reference>
			        <sender>
			            <name>". $order->billing_first_name ." ". $order->billing_last_name ."</name>
			            <email>". $order->billing_email ."</email>
			            ". $telefone_tag ." 
			        </sender>
			        <shipping>
			            <type>3</type>
			            <address>
			                <country>BRA</country>
			                <postalcode>". $order->billing_postcode ."</postalcode>
			                <city>". $order->billing_city ."</city>
			                <state>". $order->billing_state ."</state>
			            </address>
			        </shipping>
			    </checkout>";
			
			
			if ($this->debug=='yes') $this->log->add( $this->id, "Pacote XML gerado: \n". $xml );
			
			
			
			$curl = curl_init($url);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/xml; charset=UTF-8"));
			curl_setopt($curl, CURLOPT_POSTFIELDS, $xml);
			$xml= curl_exec($curl);
			curl_close($curl);
			
			
			
			if ($this->debug=='yes') $this->log->add( $this->id, "Requisição cURL: \n". $xml );
			
			
			
			if($xml == 'Unauthorized'){
		    	$woocommerce->add_error(__('There was an error during the payment process, unauthorized.', 'WC_PagSeguro'));
		    	
		    	if ($this->debug=='yes') $this->log->add( $this->id, __('There was an error during the payment process, unauthorized. '. print_r( $xml, true ), 'WC_PagSeguro') );
		    	
		    	$woocommerce->session->PSerro = "1 Requsição não autorizada";
			    return false;
			}
			
			$xml= simplexml_load_string($xml);
			
			if(count( $xml->error->code ) > 0){
				
				if ($this->debug=='yes') $this->log->add( $this->id, __('There was an error during the payment process, error on request token.', 'WC_PagSeguro'));
				
		    	$woocommerce->add_error(__('There was an error during the payment process, error on request token.', 'WC_PagSeguro'));
		    	$woocommerce->session->PSerro = "2 O valor retornado pela transação é nulo <!-- ". nl2br( print_r( $xml, true )) ."-->";
			    return false;
			}else{
				
				
				/**
				 * Write token to accept pay in other time 
				 * this is necessary why in PagSeguro for each 
				 * request added a intention of payment  
				 * unique by order.
				 */
				add_post_meta( $order->id, 'pagseguro_token', (string) $xml->code, true );
				
				
				return 'https://pagseguro.uol.com.br/v2/checkout/payment.html?code='. $xml->code .'&order='. $order->id;
			}
			
		}
		
		
		
		


		/**
		 * Add an error message
		 */
		public function add_error( $message ) {
			if ( function_exists( 'wc_add_notice' ) ) {
				wc_add_notice( $message, 'error' );
			} else {
				global $woocommerce;

				$woocommerce->add_error( $message );
			}
		}
	
	
	}

}

function add_pagseguro_gateway( $methods ) {
    $methods[] = 'WC_Gateway_Pagseguro'; return $methods;
}